package com.dxhy.erp.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.dxhy.common.constant.DbConstant;
import com.dxhy.common.datasource.config.DynamicContextHolder;
import com.dxhy.common.utils.R;
import com.dxhy.erp.dao.*;
import com.dxhy.erp.entity.sdny.*;
import com.dxhy.erp.service.ImageSubmitService;
import com.dxhy.erp.service.SdnyTDxRecordInvoiceDetailSalesService;
import com.dxhy.erp.utils.JsonUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.stream.Collectors;

@Service
@Transactional(rollbackFor = Exception.class)
public class ImageSubmitServiceImpl implements ImageSubmitService {

    @Autowired
    private ImageSubmitDao imageSubmitDao;

    @Autowired
    private SdnyTDxRecordInvoiceDao tdxRecordInvoiceDao;

    @Autowired
    private SdnyTDxRecordInvoiceDetailDao detailDao;


    @Autowired
    private SdnyTDxSaleRecordInvoiceDao sdnyTDxSaleRecordInvoiceDao;

    @Autowired
    private SdnyTDxRecordInvoiceDetailSalesService sdnyTDxRecordInvoiceDetailSalesService;

    @Override
    public R salesInvoiceSubmit(SNRequestObject params) {
        List<String> uuidList = new ArrayList<>();
        Map<String, SNInvoice> paramsInvoiceMap = new HashMap<>();
        params.getInvoiceInfoList().stream().forEach(item->{
            String uuid = (StringUtils.isBlank(item.getInvoiceCode()) ? "" : item.getInvoiceCode()) + item.getInvoiceNumber();
            uuidList.add(uuid);
            paramsInvoiceMap.put(uuid, item);
        });
        DynamicContextHolder.push(DbConstant.BUSINESS_WRITE);
        QueryWrapper<SdnyTDxRecordInvoice> queryWrapper = new QueryWrapper<>();
        queryWrapper.in("uuid", uuidList);
        List<SdnyTDxRecordInvoice> invoicesList = tdxRecordInvoiceDao.selectList(queryWrapper);
        if (invoicesList == null || invoicesList.size() == 0) {
            return R.error(500, "未查询到发票信息，请重新查验");
        }
        if (invoicesList.size() != uuidList.size()) {
            StringBuffer msg = new StringBuffer();
            Map<String, SdnyTDxRecordInvoice> invoiceMap = invoicesList.stream().collect(Collectors.toMap(SdnyTDxRecordInvoice::getUuid, v->v));
            uuidList.stream().forEach(uuid->{
                if (!invoiceMap.containsKey(uuid)) {
                    SNInvoice invoice = paramsInvoiceMap.get(uuid);
                    String errorMsg = "发票代码:"+invoice.getInvoiceCode()+",发票号码:"+invoice.getInvoiceNumber()+";";
                    msg.append(errorMsg);
                }
            });
            String errorMsg = msg.toString();
            return R.error(500, "未查询到如下发票，请重新查验"+errorMsg);
        }

        DynamicContextHolder.push(DbConstant.DETAIL_WRITE);
        QueryWrapper<SdnyTDxRecordInvoiceDetail> detailQueryWrapper = new QueryWrapper<>();
        detailQueryWrapper.in("uuid", uuidList);
        List<SdnyTDxRecordInvoiceDetail> detailList = detailDao.selectList(detailQueryWrapper);
        Map<String, List<SdnyTDxRecordInvoiceDetail>> detailiMap = new HashMap<>();
        detailList.stream().forEach(item->{
            List<SdnyTDxRecordInvoiceDetail> list = null;
            if (detailiMap.containsKey(item.getUuid())) {
                list = detailiMap.get(item.getUuid());
                list.add(item);
                detailiMap.put(item.getUuid(), list);
            } else {
                list = new ArrayList<>();
                list.add(item);
                detailiMap.put(item.getUuid(), list);
            }
        });
        invoicesList.stream().forEach(item->{
            item.setImageId(params.getImageId());
            item.setBzdh(params.getBillNum());
            item.setBzr(params.getJbrUserName());
        });
        List<SdnyTDxSaleRecordInvoice> salesInvoiceList = new ArrayList<>();
        invoicesList.stream().forEach(invoice->{


            DynamicContextHolder.push(DbConstant.BUSINESS_WRITE);
            SdnyTDxSaleRecordInvoice saleRecordInvoice = sdnyTDxSaleRecordInvoiceDao.selectOne(new QueryWrapper<SdnyTDxSaleRecordInvoice>().eq("uuid", invoice.getUuid()));
            List<SdnyTDxRecordInvoiceDetailSales> salesDetailList = new ArrayList<>();
            detailiMap.get(invoice.getUuid()).stream().forEach(item->{
                SdnyTDxRecordInvoiceDetailSales salesDetail = JsonUtils.getInstance().parseObject(JSONObject.toJSONString(item), SdnyTDxRecordInvoiceDetailSales.class);
                salesDetail.setId(null);
                salesDetailList.add(salesDetail);
            });
            if (saleRecordInvoice == null) {
                // 新增
                saleRecordInvoice = JsonUtils.getInstance().parseObject(JSONObject.toJSONString(invoice), SdnyTDxSaleRecordInvoice.class);
                saleRecordInvoice.setId(null);
                DynamicContextHolder.push(DbConstant.BUSINESS_WRITE);
                sdnyTDxSaleRecordInvoiceDao.insert(saleRecordInvoice);
                DynamicContextHolder.push(DbConstant.DETAIL_WRITE);
                sdnyTDxRecordInvoiceDetailSalesService.getBaseMapper().delete(new UpdateWrapper<SdnyTDxRecordInvoiceDetailSales>().eq("uuid", invoice.getUuid()));
                sdnyTDxRecordInvoiceDetailSalesService.saveBatch(salesDetailList);
            } else {
                // 修改
                saleRecordInvoice.setImageId(invoice.getImageId());
                saleRecordInvoice.setBzdh(invoice.getBzdh());
                saleRecordInvoice.setBzr(invoice.getBzr());
                DynamicContextHolder.push(DbConstant.BUSINESS_WRITE);
                sdnyTDxSaleRecordInvoiceDao.updateById(saleRecordInvoice);
            }
            saleRecordInvoice.setDetailList(salesDetailList);
            salesInvoiceList.add(saleRecordInvoice);
        });

        return R.success();
    }

    @Override
    public R salesInvoiceRollback(SNRequestObject params) {
        return null;
    }

    /**
     * 推送SAP
     * @param invoiceList
     * @param flag true提交 false撤回
     * @return
     */
    public R pushSap(List<SdnyTDxSaleRecordInvoice> invoiceList, Boolean flag) {



        return R.success();
    }
}
